@@ -6,7 +6,7 @@
 Plots
 
     plotool:
-        figure, set_border, set_ax,
+        figure, set_border, set_clib, set_ax,
         plot, set_font, save, show
     pplot(plotool):
         add_plot
@@ -20,14 +20,23 @@ from scipy import optimize
 import matplotlib.colors as mplc
 import matplotlib.pyplot as plt
 
+## Local
+from utilities import merge_aliases
+
 # cmap = mpl.cm.viridis
 # norm = mpl.colors.Normalize(vmin=0, vmax=1)
 sizeXS, sizeS, sizeM = 4, 6, 8
 sizeL, sizeXL, sizeXXL = 10, 12, 14
 
+##-----------------------------------------------
+##
+##            <plotool> based tools
+##
+##-----------------------------------------------
+
 class plotool:
     '''
-    PLOT tOOL
+    plot Tool
     '''
     def __init__(self, x=np.zeros(2), y=np.zeros(2)):
         
@@ -35,8 +44,6 @@ class plotool:
         self.x = x
         self.y = y
 
-        self.ax = None
-
     def figure(self, figsize=None, figint=True,
                nrows=1, ncols=1):
         
@@ -60,6 +67,18 @@ class plotool:
         plt.subplots_adjust(left=left, bottom=bottom,
             right=right, top=top, wspace=wspace, hspace=hspace)
 
+    def set_clib(self, clib):
+        if clib=='base':
+            self.clib = list(mplc.BASE_COLORS) # 8 colors
+        elif clib=='tableau':
+            self.clib = list(mplc.TABLEAU_COLORS) # 10 colors
+        elif clib=='ccs4' or clib=='x11':
+            self.clib = list(mplc.CSS4_COLORS)
+        elif clib=='xkcd':
+            self.clib = list(mplc.XKCD_COLORS)
+        else:
+            self.clib = clib
+        
     def set_ax(self, xlog=False, ylog=False,
                basex=10, basey=10, nonposx='sym', nonposy='sym',
                xlim=(None,None), ylim=(None,None),
@@ -78,31 +97,42 @@ class plotool:
                 self.ax.set_yscale('symlog',base=basey)
             else:
                 self.ax.set_yscale('log',base=basey,nonpositive=nonposy)
-                
-        self.ax.set_xlim(xlim[0], xlim[1])
-        self.ax.set_ylim(ylim[0], ylim[1])
+
+        if xlim[0]!=None or xlim[1]!=None:
+            self.ax.set_xlim(xlim[0], xlim[1])
+        if ylim[0]!=None or ylim[1]!=None:
+            self.ax.set_ylim(ylim[0], ylim[1])
 
         # self.ax.set_xticks()
         # self.ax.set_yticks()
         # self.ax.set_xticklabels()
         # self.ax.set_yticklabels()
-        
-        self.ax.set_xlabel(xlab)
-        self.ax.set_ylabel(ylab)
-        self.ax.set_title(title)
-        self.ax.legend(loc=legend)
+
+        if xlab is not None:
+            self.ax.set_xlabel(xlab)
+        if ylab is not None:
+            self.ax.set_ylabel(ylab)
+        if title is not None:
+            self.ax.set_title(title)
+        if legend is not None:
+            self.ax.legend(loc=legend)
         self.legend = legend
     
     def plot(self, nrow=1, ncol=1,
              x=None, y=None, xerr=None, yerr=None,
-             xlim=(None, None), ylim=(None, None),
-             xlog=False, ylog=False,
-             basex=10, basey=10, nonposx='sym', nonposy='sym',
-             fmt='', c=None, ls=None, lw=None,
-             ecolor=None, elinewidth=None, capsize=None, barsabove=False,
-             xlab=None, ylab=None,
-             label=None, legend=None, title=None, mod='CA'):
+             fmt='', capsize=None, barsabove=False, # errorbar kw
+             ecolor=None, ec=None, elinewidth=None, elw=None, # errorbar kw
+             mod='CA', **kwargs):
+        '''
+        Like set_ax(), this is a clump operation.
+        The idea is to all set in one command,
+        while each single operation should also be valid.
+        '''
 
+        ## kw aliases
+        ec = merge_aliases(None, ecolor=ecolor, ec=ec)
+        elw = merge_aliases(None, elinewidth=elinewidth, elw=elw)
+        
         if x is None:
             x = self.x
         else:
@@ -111,19 +141,17 @@ class plotool:
             y = self.y
         else:
             self.y = y
-            
+
+        ## CA: Cartesian using matplotlib.pyplot.errorbar
         if mod=='CA':
             if self.nrows!=1 or self.ncols!=1:
                 self.ax = self.axes[nrow-1,ncol-1]
                 
             self.markers, self.caps, self.bars = self.ax.errorbar(
                 x=x, y=y, yerr=yerr, xerr=xerr,
-                fmt=fmt, c=c, ls=ls, lw=lw,
-                ecolor=ecolor, elinewidth=elinewidth,
-                capsize=capsize, barsabove=barsabove, label=label)
-            
-            self.set_ax(xlog, ylog, basex, basey, nonposx, nonposy,
-                        xlim, ylim, xlab, ylab, legend, title)
+                fmt=fmt, ecolor=ec, elinewidth=elw,
+                capsize=capsize, barsabove=barsabove,
+                **kwargs)
         
         else:
             print('*******************')
@@ -159,61 +187,69 @@ class plotool:
         plt.ioff()
         plt.show()
 
-##-----------------------------------------------
-
-##                    sub-classes
-
-##-----------------------------------------------
-
 class pplot(plotool):
     '''
     Uni-frame plot (1 row * 1 col)
     '''
-    def __init__(self, x, y, xerr=None, yerr=None,
-                 xlim=(None, None), ylim=(None,None),
-                 xlog=None, ylog=None,
-                 basex=10, basey=10, nonposx='sym', nonposy='sym',
-                 fmt='', c=None, ls=None, lw=.5, ec='r', elw=.8,
-                 capsize=None, barsabove=False, label=None,
-                 xlab='X', ylab='Y', legend=None,
-                 title='Untitled', figsize=None, figint=False,
-                 left=.1, bottom=.1, right=.99, top=.9,
-                 wspace=None, hspace=None, clib='base'):
+    def __init__(self, x=None, y=None, xerr=None, yerr=None,
+                 fmt='', capsize=None, barsabove=False, # errorbar kw
+                 ecolor=None, ec=None, elinewidth=None, elw=None, # errorbar kw
+                 figsize=None, figint=False, # figure kw
+                 left=.1, bottom=.1, right=.99, top=.9, # set_border kw
+                 wspace=None, hspace=None, # set_border kw
+                 xlim=(None, None), ylim=(None,None), xlog=None, ylog=None, # set_ax kw
+                 basex=10, basey=10, nonposx='sym', nonposy='sym', # set_ax kw
+                 xlab='X', ylab='Y', legend=None, title='Untitled', # set_ax kw
+                 clib='base', c=None, **kwargs):
         super().__init__(x, y)
 
-        ## Named color lib
-        if clib=='base':
-            self.clib = list(mplc.BASE_COLORS) # 8 colors
-        elif clib=='tableau':
-            self.clib = list(mplc.TABLEAU_COLORS) # 10 colors
-        elif clib=='ccs4' or clib=='x11':
-            self.clib = list(mplc.CSS4_COLORS)
-        elif clib=='xkcd':
-            self.clib = list(mplc.XKCD_COLORS)
-        else:
-            raise ValueError('Unname colors! ')
-
-        self.figure(figsize, figint)
         self.iplot = 0
 
+        ## kw aliases
+        ec = merge_aliases(None, ecolor=ecolor, ec=ec)
+        elw = merge_aliases(None, elinewidth=elinewidth, elw=elw)
+
+        ## Auto color
+        self.set_clib(clib)
+        if c is None:
+            c = self.clib[self.iplot]
+
+        ## Init figure
+        self.figure(figsize, figint)
+
+        ## set_border
         self.set_border(left=left, bottom=bottom,
             right=right, top=top, wspace=wspace, hspace=hspace)
 
-        if c is None:
-            c = self.clib[self.iplot]
-        self.plot(x, y, xerr=xerr, yerr=yerr,
-                  fmt=fmt, c=c, ls=ls, lw=lw, ecolor=ec, elinewidth=elw,
-                  capsize=capsize, barsabove=barsabove, label=label)
+        ## plot
+        self.plot(x=x, y=y, xerr=xerr, yerr=yerr,
+                  fmt=fmt, ec=ec, elw=elw, # errorbar kw
+                  capsize=capsize, barsabove=barsabove, # errorbar kw
+                  c=c, **kwargs)
 
+        ## set_ax
         self.set_ax(xlog, ylog, basex, basey, nonposx, nonposy,
                     xlim, ylim, xlab, ylab, legend, title)
-
+        
         self.set_font()
 
     def add_plot(self, x=None, y=None, xerr=None, yerr=None,
-                 fmt='', c=None, ls=None, lw=.5, ec='r', elw=.8,
-                 capsize=None, barsabove=False, label=None):
+                 fmt='', capsize=None, barsabove=False, # errorbar opt
+                 ecolor=None, ec=None, elinewidth=None, elw=None, # errorbar opt
+                 c=None, **kwargs):
 
+        self.iplot += 1
+
+        ## kw aliases
+        ec = merge_aliases(None, ecolor=ecolor, ec=ec)
+        elw = merge_aliases(None, elinewidth=elinewidth, elw=elw)
+
+        ## Auto color
+        if self.iplot==len(self.clib):
+            self.iplot = 0
+        if c is None:
+            c = self.clib[self.iplot]
+        
         if x is None:
             x = self.x
         else:
@@ -222,13 +258,12 @@ class pplot(plotool):
             y = self.y
         else:
             self.y = y
-            
-        self.iplot += 1
-        if c is None:
-            c = self.clib[self.iplot]
 
-        self.plot(x, y, xerr=xerr, yerr=yerr,
-                  fmt=fmt, c=c, ls=ls, lw=lw, ecolor=ec, elinewidth=elw,
-                  capsize=capsize, barsabove=barsabove, label=label)
+        ## plot
+        self.plot(x=x, y=y, xerr=xerr, yerr=yerr,
+                  fmt=fmt, ec=ec, elw=elw, # errorbar kw
+                  capsize=capsize, barsabove=barsabove, # errorbar kw
+                  c=c, **kwargs)
 
-        self.set_ax(legend=self.legend)
+        ## Add legend
+        self.ax.legend(loc=self.legend)
